import React__default,{useState,useEffect}from'react';import {Spin,message}from'antd';import TimerWidget from'../HeaderTimerHandler/TimerWidget/index.js';import {useDispatch,useSelector}from'react-redux';import {currentTimerSelector,setCurrentTimerAction}from'@admin-layout/timetracker-core';import WeeklyRecord from'./WeeklyRecord/index.js';import'tslib';import {calcDurationTimeMilliSec}from'../../../utils/timeConverter.js';import {useGetTimeRecordsQuery,useCreateTimeRecordMutation,useUpdateTimeRecordMutation}from'@admin-layout/timetracker-core/lib/generated/react-hooks';import {useCreatePermissions}from'../../hooks/index.js';import moment from'moment';import {omit}from'lodash';const getDate = (startDate, endDate) => {
    if (moment(startDate).format("ddd, Do MMM'YY") === moment(endDate).format("ddd, Do MMM'YY")) {
        return moment(startDate).format("ddd, Do MMM'YY");
    }
    return `${moment(startDate).format("ddd, Do MMM")} - ${moment(endDate).format("ddd, Do MMM'YY")}`;
};
const groupTimeRecords = (timeRecords) => {
    if (!timeRecords)
        return [];
    timeRecords.sort((a, b) => {
        if (moment(a.endTime) < moment(b.endTime))
            return 1;
        else if (moment(a.endTime) > moment(b.endTime))
            return -1;
        else
            return 0;
    });
    let groupedDates = {};
    for (let i = 0; i < timeRecords.length; i++) {
        if (timeRecords[i].startTime && timeRecords[i].endTime) {
            const date = moment(timeRecords[i].endTime);
            const dateStr = moment(date).startOf('week').format();
            if (!groupedDates.hasOwnProperty(dateStr)) {
                groupedDates[dateStr] = [];
            }
            groupedDates[dateStr].push(timeRecords[i]);
        }
    }
    let groupedDateArray = [];
    for (let key of Object.keys(groupedDates)) {
        let timeArray = groupedDates[key];
        const startDate = moment(key).startOf('week').format('ddd, MMM DD');
        const endDate = moment(key).endOf('week').format('ddd, MMM DD');
        let totalDurations = 0;
        let groupedDays = {};
        timeArray.forEach((data) => {
            totalDurations += calcDurationTimeMilliSec(data.startTime, data.endTime);
            const dateString = getDate(data.startTime, data.endTime);
            if (!groupedDays.hasOwnProperty(dateString)) {
                groupedDays[dateString] = [];
            }
            groupedDays[dateString].push(data);
        });
        let groupedDaysArray = [];
        for (let dayKey of Object.keys(groupedDays)) {
            let duration = 0;
            groupedDays[dayKey].forEach((day) => {
                duration += calcDurationTimeMilliSec(day.startTime, day.endTime);
            });
            groupedDaysArray.push({
                date: dayKey,
                duration,
                days: groupedDays[dayKey],
            });
        }
        groupedDateArray.push({
            dates: `${startDate} - ${endDate}`,
            duration: totalDurations,
            groupedDays: groupedDaysArray,
        });
    }
    return groupedDateArray;
};
const TrayTimerHandler = (props) => {
    const dispatch = useDispatch();
    const currentTimer = useSelector(currentTimerSelector);
    const userId = useSelector((state) => state.user.auth0UserId);
    const orgName = useSelector((state) => state.platform.orgName);
    const [timeRecords, setTimeRecords] = useState([]);
    const [loading, setLoading] = useState(true);
    const [startTimer, setStartTimer] = useState(false);
    const [isPause, setIsPause] = useState(false);
    const [currentTimeRecord, setCurrentTimeRecord] = useState({
        startTime: null,
        endTime: null,
        isBillable: false,
        description: '',
        projectId: '',
        taskName: '',
    });
    const { data, loading: loadingData, refetch } = useGetTimeRecordsQuery({ variables: { userId } });
    const [createMutation] = useCreateTimeRecordMutation();
    const [updateMutation] = useUpdateTimeRecordMutation();
    const { self: createPermit } = useCreatePermissions();
    useEffect(() => {
        setTimeout(() => {
            setLoading(false);
        }, 10);
    }, []);
    useEffect(() => {
        refetch();
    }, [orgName]);
    useEffect(() => {
        if (data) {
            setTimeRecords(groupTimeRecords(data.getTimeRecords));
        }
    }, [data]);
    useEffect(() => {
        if (currentTimer.isPause) ;
        else if (currentTimer && (currentTimer.startTime === null || currentTimer.endTime !== null)) {
            if (startTimer) {
                resetTimerValues();
                refetch();
            }
        }
        else {
            setCurrentTimeRecord(omit(currentTimer, ['isPause', 'isTimeout']));
            setStartTimer(true);
        }
        if (currentTimer.isPause !== isPause) {
            setIsPause(currentTimer.isPause);
        }
    }, [currentTimer]);
    const createTimeRecord = (request) => {
        if (createPermit !== "Allow" /* Allow */) {
            message.warning('Permission Not Allowed');
            return;
        }
        createMutation({ variables: { request } })
            .then(() => {
            message.info('TimeRecord created');
        })
            .catch((error) => {
            message.error(error.message);
        });
    };
    const updateTimeRecord = (recordId, request) => {
        updateMutation({ variables: { recordId, request } })
            .then(() => {
            message.success('TimeRecord Updated');
        })
            .catch((error) => {
            message.error(error.message);
        });
    };
    const resetTimerValues = () => {
        setStartTimer(false);
        const defaultPlayingRecord = {
            startTime: null,
            description: '',
            endTime: null,
            isBillable: false,
            projectId: '',
            taskName: '',
        };
        setCurrentTimeRecord(Object.assign({}, defaultPlayingRecord));
        dispatch(setCurrentTimerAction(Object.assign(Object.assign({}, defaultPlayingRecord), { isPause: false, isTimeout: false })));
    };
    const saveCurrentTimeRecord = () => {
        const endTime = moment();
        const startTime = moment(currentTimeRecord.startTime);
        const newTimeRecord = {
            userId: userId,
            startTime: startTime,
            endTime: endTime,
            description: currentTimeRecord.description,
            projectId: currentTimeRecord.projectId,
            isBillable: currentTimeRecord.isBillable,
            taskName: currentTimeRecord.taskName,
        };
        if (currentTimeRecord.id === undefined || currentTimeRecord.id === '')
            createTimeRecord(newTimeRecord);
        else
            updateTimeRecord(currentTimeRecord.id, newTimeRecord);
    };
    const updatePlayingTimeRecord = (timeRecord) => {
        setCurrentTimeRecord(timeRecord);
        if (currentTimeRecord.id !== undefined && currentTimeRecord.id !== '') {
            updateTimeRecord(timeRecord.id, Object.assign({}, omit(timeRecord, ['__typename', 'id'])));
        }
    };
    const onStartTimer = () => {
        if (currentTimeRecord.startTime === null) {
            const newTimeRecord = Object.assign(Object.assign({}, currentTimeRecord), { userId, startTime: moment(), endTime: null });
            createTimeRecord(newTimeRecord);
        }
        else {
            dispatch(setCurrentTimerAction(Object.assign(Object.assign({}, currentTimer), { isPause: false })));
        }
    };
    const onStopTimer = () => {
        saveCurrentTimeRecord();
    };
    const onPauseTimer = () => {
        dispatch(setCurrentTimerAction(Object.assign(Object.assign({}, currentTimer), { isPause: true })));
    };
    let tRecords = [];
    if (!loading) {
        tRecords = timeRecords;
    }
    return (React__default.createElement("div", null,
        React__default.createElement(TimerWidget, { onClose: () => { }, onStart: onStartTimer, onPause: onPauseTimer, onStop: onStopTimer, isPause: isPause, trackStarted: startTimer, currentTimeRecord: currentTimeRecord, updatePlayingTimeRecord: updatePlayingTimeRecord }),
        React__default.createElement(Spin, { spinning: loading || loadingData },
            React__default.createElement("div", { className: "time-logs" }, tRecords.map((log, index) => React__default.createElement(WeeklyRecord, { key: index, log: log }))))));
};export default TrayTimerHandler;//# sourceMappingURL=index.js.map
